#!/bin/bash

######################################################################
############# G15-Simple v1.5  (released under GPL 2.0) ##############
######################################################################
# Author:                         Jocke "firetech" Andersson         #
# (Loosely based on 'G15 Amarok Plugin Plus v1.0' by Robert Riemann) #
# Report bugs and requests to:    jocke@barbanet.com                 #
# Requires:                       g15composer, iconv, amarok, dcop   #
######################################################################
# The script shows the meta info to the actual track of Amarok on    #
# the G15 LCD screen: title, artist, album and length                #
######################################################################

# Check installation of needed apps
function checkBinary {
	if [ -z $(which $1) ]; then
		echo "${1} could not be found in your PATH. Aborting." >&2
		exit 1
	fi
}
checkBinary g15composer
checkBinary iconv
checkBinary dcop

# Make sure amarok is running and that we can dcop to it
dcop amarok player status > /dev/null 2>&1
if [ "$?" != "0" ]; then
	echo "Could not dcop to Amarok (Is it running?) Aborting." >&2
	exit 1
fi

# Set fifo pipe to use
pipe='/tmp/ft-g15-amarok'

# Check for an already running g15composer with our pipe - Normally there is'nt one
function getComposer {
	pgrep -f -x "g15composer ${pipe}"
}
function checkComposer {
	local pid=$(getComposer)
	if [ -z "${pid}" ]; then
		[ -e "${pipe}" -a ! -p "${pipe}" ] && rm -f $pipe
		g15composer $pipe &
		while [ ! -p "${pipe}" ]; do
			sleep 1
		done
	fi
}

# Get control about the signals to stop the g15composer and remove the pipe
function exit_cleanly {
	local pid=$(getComposer)
	[ -n "${pid}" ] && kill $pid
	[ -e "${pipe}" ] && rm -f $pipe
	exit 0
}
trap "exit_cleanly" EXIT HUP INT QUIT FPE KILL TERM

# Construct three dots at specified coordinates (X Y)
function make_dots {
	local y=$(($1+5))
	cat << EOF

PS 152 ${y} 1
PS 155 ${y} 1
PS 158 ${y} 1
EOF
}

# Main loop
if [ $(dcop amarok player status) -gt 0 ]; then
	amarok_stdin="trackChange"
else
	amarok_stdin="engineStateChange: empty"
fi
while true; do
	do_print=0
	
	# Show the meta infos if amarok is'nt stopped
	if [ "${amarok_stdin}" = "trackChange" -o "${amarok_stdin}" = "engineStateChange: paused" \
		-o "${amarok_stdin}" = "engineStateChange: playing" ]; then
		# Prepares the title, artist and album
		
		albumline="DL 0 20 159 20 1
"
		albumdots=""
		offset=0
		album=$(dcop amarok player album)
		if [ -z "${album}" ]; then
			albumline=""
			offset=6
		elif [ ${#album} -gt 32 ]; then
			album="${album:0:30}  "
			albumdots=$(make_dots 23)
		fi

		artistdots=""
		artist=$(dcop amarok player artist)
		if [ ${#artist} -gt 32 ]; then
			artist="${artist:0:30}  "
			artistdots=$(make_dots $offset)
		fi
		
		titledots=""
		title=$(dcop amarok player title)
		if [ ${#title} -gt 32 ]; then
			title="${title:0:30}  "
			titledots=$(make_dots $((10+$offset)))
		fi

		length=$(dcop amarok player totalTime)

		if [ $(dcop amarok player status) = 2 ]; then
			icons="PO 1 35 8 8 \"1100000011110000111111001111111111111111111111001111000011000000\"
PO 151 35 8 8 \"1100000011110000111111001111111111111111111111001111000011000000\""
		else
			icons="PF 1 35 3 42 1
PF 6 35 8 42 1
PF 151 35 153 42 1
PF 156 35 158 42 1"
		fi

		do_print=1
	
	# Shows the amarok stopped status screen
	elif [ "${amarok_stdin}" = "engineStateChange: empty" ]; then
		artist=""
		artistdots=""
		title="No track playing"
		titledots=""
		albumline=""
		offset=0
		album=""
		albumdots=""
		length=""
		icons="PF 1 35 8 42 1
PF 151 35 158 42 1"
		do_print=1
	fi
	
	# Draw the screen
	if [ $do_print = 1 ]; then
		checkComposer
		iconv -t ISO-8859-1 -c > $pipe << EOF
MC 1
TO 0 ${offset} 1 1 "${artist}"${artistdots}
TO 0 $((10+$offset)) 1 1 "${title}"${titledots}
${albumline}TO 0 23 1 1 "${album}"${albumdots}
DL 0 32 159 32 1
TO 0 36 0 1 "${length}"
${icons}
MC 0
EOF
	fi

	read amarok_stdin
done
